@@ -99,6 +99,11 @@ void parseirc(char *serveroutput)
 	if(!strcasecmp(command, "004")) {
 		return;
 	}
+	if(!strcasecmp(command, "005")) {
+		/* at least undernet and ircnet return the
+		   server flags this way */
+		return;
+	}
 	if(!strcasecmp(command, "221")) {
 		return;
 	}
@@ -258,6 +263,10 @@ void parseirc(char *serveroutput)
 		oer_debug(OER_DEBUG_INFO, "parseirc(404)->%s\n", restptr);
 		return;
 	}
+	if(!strcasecmp(command, "422")) {
+		oer_debug(OER_DEBUG_INFO, "parseirc(422)->%s\n", restptr);
+		return;
+	}
 	if(!strcasecmp(command, "433")) {
 		oer_debug(OER_DEBUG_INFO, "parseirc(433)->our nickname %s is already in use, will try altnick %s\n", mystate->nick, mystate->altnick);
 		strcpy(altnick, mystate->nick);
@@ -534,7 +543,11 @@ void parseirc(char *serveroutput)
 			return;
 		}
 		/* KICK doesn't tell the host for the person that was kicked */
-		cu = getcuptr(channel, target);
+		if((cu = getcuptr(channel, target)) == NULL) {
+                        /* we don't know the host, this is necessary because of
+                           bogus/lagged server/service messages */
+                        return;
+                }
 		oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->%s!%s off %s by %s!%s\n", target, cu->host, channel, nick, host);
 		addnewpart(channel, mystate->now, target, cu->host);
 		if(!userleft(channel, target, cu->host)) {
@@ -551,7 +564,11 @@ void parseirc(char *serveroutput)
 		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
 			return;
 		}
-		strncpy(host, prefix + 1 + tpos, HOSTLEN);
+		if(strlen(prefix + 1 + tpos)) {
+			strncpy(host, prefix + 1 + tpos, HOSTLEN);
+		} else {
+			memset(host, 0, HOSTLEN);
+		}
 		strncpy(channel, params, CHANLEN);
 		restpos = 0;
 		if((restpos = parse(restptr, restpos, " ", mode, CHANLEN, 0)) < 0) {
@@ -1010,6 +1027,27 @@ void parsecommand(char *target, char *nick, char *host, char *commandline)
 			addnewtimed(mystate->now, OER_TIMED_IRC_COMMAND, OER_TIMED_IRC_COMMAND_DUMMY, outstring, NULL, NULL, NULL);
 		}
 		return;
+	case OER_WHICHCOMMAND_RANDOM_BANKICK:
+		oer_debug(OER_DEBUG_INFO, "parsecommand->RANDOM_BANKICK\n");
+		proceed = 0;
+		if(isadmin(channel, host)) {
+			proceed = 1;
+		}
+		if(isop(channel, host) && hasuserflags(channel, host, "r")) {
+			proceed = 1;
+		}
+		if(!proceed) {
+			return;
+		}
+		if(amiop(channel)) {
+			u = getrandomuser(channel);
+			if(u == NULL) {
+				return;
+			}
+			snprintf(outstring, WRITE_BUFFER_LENGTH, "random ban kick by %s\n", nick);
+			bankickuser(channel, u->nick, outstring);
+		}
+		return;
 	case OER_WHICHCOMMAND_SYNC:
 		oer_debug(OER_DEBUG_INFO, "parsecommand->SYNC\n");
                 if(!isatleastopnow(channel, nick, host)) {
